
# example for reading csv table with #N/A as NA-flags
# nordpool = read.table(file="nordpool spot 1996-2011.csv",header=TRUE,sep=",",na.strings=c("#N/A"))

replace_NA = function(data, method="mean", precision=2)
{
  nas = which(is.na(data))   # find na-values in data
  n = length(nas)
  if(n>0)
  {
    for (i in 1:n)
    {
      j=1
      while ((i+j)<=n & nas[i+j]==nas[i]+j){
        j=j+1                       # find how many consecutive NAs we have here
      }

      start = data[nas[i]-1]
      end = data[nas[i]+j]
      step = (end-start) / (j+1)     # mean step value at NA sequence

      for (k in 1:j)
      {
          data[nas[i+k-1]] = start + step*k
      }
    }
  }

  # truncate trailing NA values
  while(is.na(tail(data,1)))
    data = head(data,length(data)-1)

  round(data,precision)
}

detectoutliers = function(diff,w=3)
{
  n = length(diff)
  sorted = sort(diff)
  n1 = floor(n/4)
  n3 = n+1-n1
  Q1 = sorted[n1]
  Q3 = sorted[n3]
  IQR = Q3-Q1
  limits = c(Q1-w*IQR, Q3+w*IQR)

  indexlist = c()
  for(i in 1:n)
  {
    if(diff[i]>limits[2] | diff[i]<limits[1])
    {
      indexlist = c(indexlist, i+1)
    }
  }
  print(paste("# of outliers: ", length(indexlist)))
  indexlist
}

removeoutliers = function(data, w=3)
{
  diff = diff(log(data))
  outliers = detectoutliers(diff,w)
  data[outliers] = NA
  data = replace_NA(data)
  data
}

